"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const get_current_browser_window_state_1 = require("./get-current-browser-window-state");
const emulate_device_1 = require("../page-actions/emulate-device");
function tryLaunchBrowser(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const maxRetries = 3;
        if (options.defaultViewport === undefined) {
            options.defaultViewport = null;
        }
        for (let index = 0; index <= maxRetries; index++) {
            try {
                const browser = yield require('puppeteer-core').launch(options);
                return browser;
            }
            catch (error) {
                if (index >= maxRetries) {
                    throw error;
                }
                if (error.message &&
                    typeof error.message === 'string' &&
                    error.message.includes('Failed to launch chrome') &&
                    error.message.includes('Inconsistency detected by ld.so')) {
                    // eslint-disable-next-line no-console
                    console.warn(`an error has occured while launching the browser by puppeteer:`);
                    if (error.message) {
                        // eslint-disable-next-line no-console
                        console.warn(`${error.message}`);
                    }
                    // eslint-disable-next-line no-console
                    console.warn(`retrying to launch the browser ...`);
                    continue;
                }
                if (error.message &&
                    typeof error.message === 'string' &&
                    error.message.includes('Failed to launch chrome')) {
                    // no need to retry: path to chrome or firefox is incorrect
                    throw error;
                }
            }
        }
        throw new Error('Cannot launch browser');
    });
}
function launchBrowserWithMaxSizeWindow(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const isHeadless = options.headless;
        const newOptions = Object.assign(Object.assign({}, options), { headless: false });
        const browser = yield tryLaunchBrowser(newOptions);
        const page = yield browser.newPage();
        const windowState = yield get_current_browser_window_state_1.getCurrentBrowserWindowState(page);
        newOptions.headless = isHeadless || false;
        newOptions.args = newOptions.args || [];
        newOptions.args = newOptions.args.filter((arg) => !arg.startsWith('--window-size'));
        newOptions.args.push(`--window-size=${windowState.screen.availWidth},${windowState.screen.availHeight}`);
        newOptions.defaultViewport = null;
        if (options.minViewPort &&
            typeof options.minViewPort.minWidth === 'number' &&
            typeof options.minViewPort.minHeight === 'number') {
            const width = windowState.screen.availWidth < options.minViewPort.minWidth
                ? options.minViewPort.minWidth
                : windowState.screen.availWidth;
            const height = windowState.screen.availHeight < options.minViewPort.minHeight
                ? options.minViewPort.minHeight
                : windowState.screen.availHeight;
            newOptions.defaultViewport = {
                width,
                height,
            };
        }
        yield browser.close();
        const maximizedBrowser = yield require('puppeteer-core').launch(newOptions);
        return maximizedBrowser;
    });
}
exports.launchBrowserWithMaxSizeWindow = launchBrowserWithMaxSizeWindow;
function launchBrowserWithEmulatedDevice(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const newOptions = Object.assign({}, options);
        const device = options.emulateDevice || emulate_device_1.defaultDevice;
        newOptions.args = newOptions.args || [];
        newOptions.args = newOptions.args.filter((arg) => !arg.startsWith('--window-size'));
        newOptions.args.push(`--window-size=${device.viewport.width},${device.viewport.height + 100}`);
        const emulatedDeviceBrowser = yield require('puppeteer-core').launch(newOptions);
        return emulatedDeviceBrowser;
    });
}
exports.launchBrowserWithEmulatedDevice = launchBrowserWithEmulatedDevice;
function launchBrowser(options) {
    return __awaiter(this, void 0, void 0, function* () {
        if (options.browserWindowShouldBeMaximized) {
            return yield launchBrowserWithMaxSizeWindow(options);
        }
        if (options.emulateDevice) {
            return yield launchBrowserWithEmulatedDevice(options);
        }
        return yield tryLaunchBrowser(options);
    });
}
exports.launchBrowser = launchBrowser;
