"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
function isVisible(selector, page) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!page) {
            throw new Error(`Error: cannot check if '${selector}' is visible because a new page has not been created`);
        }
        const elementExists = yield index_1.exists(selector, page);
        if (!elementExists) {
            return false;
        }
        try {
            const result = yield page.$eval(selector, (el) => {
                function hasVisibleBoundingBox(element) {
                    const rect = element.getBoundingClientRect();
                    return !!(rect.top || rect.bottom || rect.width || rect.height);
                }
                const style = window.getComputedStyle(el);
                if (style && style.opacity && style.opacity === '0') {
                    return false;
                }
                const isVisible = style && style.visibility !== 'hidden' && hasVisibleBoundingBox(el);
                return isVisible;
            });
            return result;
        }
        catch (error) {
            // eslint-disable-next-line no-console
            console.warn(`An internal error has occured in Puppeteer API while checking if selector '${selector}'  is visible`, error);
            return false;
        }
    });
}
exports.isVisible = isVisible;
