"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
const dom_actions_1 = require("../../dom-actions");
exports.defaultSelectOptions = {
    timeoutInMilliseconds: 30000,
};
function select(selector, labels, options, page) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!page) {
            throw new Error(`Error: cannot select option(s) '${labels}' in '${selector}' because a new page has not been created`);
        }
        yield __1.waitUntilSelectorIsVisible(selector, { timeoutInMilliseconds: options.timeoutInMilliseconds }, page);
        yield __1.waitUntilSelectorDoesNotMove(selector, {
            timeoutInMilliseconds: options.timeoutInMilliseconds,
        }, page);
        yield __1.waitUntilOptionsAreAvailable(selector, labels, {
            timeoutInMilliseconds: options.timeoutInMilliseconds,
        }, page);
        const availableOptions = yield dom_actions_1.getAllOptionsOf(selector, page);
        const tobeSelectedOptions = [...labels];
        const canSelect = tobeSelectedOptions.every((tobeSelectedOption) => {
            const match = availableOptions.find((availableOption) => availableOption.label === tobeSelectedOption);
            if (match === undefined) {
                return false;
            }
            return true;
        });
        if (!canSelect) {
            throw new Error(`Cannot select '${tobeSelectedOptions.join(',')}' in list '${selector}' because it does not match available options: '${availableOptions
                .map((o) => o.label)
                .join(',')}'`);
        }
        const optionValues = labels.map((optionLabel) => {
            const match = availableOptions.find((availableOption) => availableOption.label === optionLabel);
            if (match === undefined) {
                return optionLabel;
            }
            return match.value;
        });
        yield page.select(selector, ...optionValues);
    });
}
exports.select = select;
