"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const dom_actions_1 = require("../../dom-actions");
function waitUntilOptionsAreAvailable(selector, expectedOptionLabels, options, page) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!page) {
            throw new Error(`Error: cannot check if option(s) [${expectedOptionLabels.join(',')}] is available in '${selector}' because a new page has not been created`);
        }
        const waitInterval = 10;
        let elapsedTime = 0;
        let stabilityCounter = 0;
        const maxStabilityCounter = 7;
        // eslint-disable-next-line no-constant-condition
        while (true) {
            if (elapsedTime > options.timeoutInMilliseconds) {
                throw new Error(`Error: option(s) [${expectedOptionLabels.join(',')}] is still missing in '${selector}' after ${options.timeoutInMilliseconds} ms`);
            }
            if (stabilityCounter >= maxStabilityCounter) {
                return;
            }
            yield page.waitFor(waitInterval);
            elapsedTime += waitInterval;
            const areOptionsAvailable = yield dom_actions_1.areOptionsAvailableIn(selector, expectedOptionLabels, page);
            if (areOptionsAvailable) {
                stabilityCounter += 1;
                continue;
            }
            stabilityCounter = 0;
        }
    });
}
exports.waitUntilOptionsAreAvailable = waitUntilOptionsAreAvailable;
