"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const dom_actions_1 = require("../../dom-actions");
function waitUntilSelectorDoesNotMove(selector, options, page) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!page) {
            throw new Error(`Error: cannot wait for selector '${selector}' to not move because a new page has not been created`);
        }
        const waitIntervalForIsMovingFunc = 50;
        const waitInterval = 10;
        let elapsedTime = 0;
        let stabilityCounter = 0;
        const maxStabilityCounter = 7;
        // eslint-disable-next-line no-constant-condition
        while (true) {
            if (elapsedTime > options.timeoutInMilliseconds) {
                throw new Error(`Error: Selector '${selector}' is still moving after ${options.timeoutInMilliseconds} ms`);
            }
            if (stabilityCounter >= maxStabilityCounter) {
                return;
            }
            const isCurrentlyMoving = yield dom_actions_1.isMoving(selector, page);
            if (isCurrentlyMoving) {
                yield page.waitFor(waitInterval);
                stabilityCounter = 0;
                elapsedTime += waitIntervalForIsMovingFunc + waitInterval;
                continue;
            }
            stabilityCounter += 1;
            elapsedTime += waitIntervalForIsMovingFunc;
        }
    });
}
exports.waitUntilSelectorDoesNotMove = waitUntilSelectorDoesNotMove;
