"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
function waitUntilSelectorWithTextIsVisible(selector, text, options, page) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!page) {
            throw new Error(`Error: cannot wait for selector '${selector}' to be visible because a new page has not been created`);
        }
        try {
            yield page.waitFor(({ selector, text }) => {
                const elements = Array.from(document.querySelectorAll(selector));
                if (elements.length === 0) {
                    return false;
                }
                const foundElements = elements.filter((element) => {
                    const htmlElement = element;
                    if (htmlElement && htmlElement.innerText && htmlElement.innerText.includes(text)) {
                        return true;
                    }
                    return false;
                });
                if (foundElements.length > 0) {
                    return true;
                }
                return false;
            }, {
                hidden: false,
                visible: true,
                timeout: options.timeoutInMilliseconds,
            }, { selector, text });
        }
        catch (error) {
            if (error &&
                error.message &&
                typeof error.message === 'string' &&
                error.message.includes('timeout')) {
                throw new Error(`Error: waiting for selector '${selector}' with text '${text}' failed: timeout ${options.timeoutInMilliseconds}ms exceeded`);
            }
            throw error;
        }
    });
}
exports.waitUntilSelectorWithTextIsVisible = waitUntilSelectorWithTextIsVisible;
