"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const action = __importStar(require("../actions"));
const actions_1 = require("../actions");
const utils_1 = require("../utils");
const selector_1 = require("../selector");
exports.defaultWaitUntilOptions = {
    stabilityInMilliseconds: 300,
    throwOnTimeout: false,
    timeoutInMilliseconds: 30000,
    verbose: false,
};
exports.defaultAssertOptions = {
    timeoutInMilliseconds: 30000,
    stabilityInMilliseconds: 300,
};
var actions_2 = require("../actions");
exports.mandatoryFullPageScreenshotOptions = actions_2.mandatoryFullPageScreenshotOptions;
var selector_2 = require("../selector");
exports.SelectorController = selector_2.SelectorController;
class PuppeteerController {
    /**
     * Create a controller instance by using an existing browser and page instance
     */
    constructor(browser, page) {
        this.isExecutingActions = false;
        this.actions = [];
        this.launchOptions = {};
        this.pageErrors = [];
        this.failedRequests = [];
        this.sentRequests = [];
        if (browser && page) {
            this.browser = browser;
            this.page = page;
        }
    }
    then(onfulfilled, onrejected) {
        return __awaiter(this, void 0, void 0, function* () {
            // prettier-ignore
            return yield this.executeActions()
                .then(onfulfilled)
                .catch(onrejected);
        });
    }
    get currentPage() {
        return this.page;
    }
    get lastError() {
        return this._lastError;
    }
    getPageErrors() {
        return [...this.pageErrors];
    }
    clearPageErrors() {
        this.pageErrors = [];
    }
    getFailedRequests() {
        return [...this.failedRequests];
    }
    clearFailedRequests() {
        this.failedRequests = [];
    }
    getRequestsTo(url) {
        return [...this.sentRequests.filter((req) => req.url().includes(url))];
    }
    getLastRequestTo(url) {
        return this.sentRequests.filter((req) => req.url().includes(url)).pop();
    }
    clearRequestsTo(url) {
        this.sentRequests = [...this.sentRequests.filter((req) => !req.url().includes(url))];
    }
    executeActions() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.isExecutingActions = true;
                this._lastError = undefined;
                // eslint-disable-next-line no-constant-condition
                while (true) {
                    if (this.actions.length === 0) {
                        break;
                    }
                    const action = this.actions.shift();
                    action && (yield action());
                }
            }
            catch (error) {
                this._lastError = error;
                this.actions = [];
                this.isExecutingActions = false;
                throw error;
            }
            finally {
                this.actions = [];
                this.isExecutingActions = false;
            }
        });
    }
    launchAction(options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.launchOptions = Object.assign(Object.assign({}, this.launchOptions), options);
            this.launchOptions.executablePath = this.launchOptions.executablePath || utils_1.getChromePath();
            this.browser = yield action.launchBrowser(this.launchOptions);
        });
    }
    startNewPage() {
        return __awaiter(this, void 0, void 0, function* () {
            this.page = yield action.startNewPageInBrowser(this.browser, this.launchOptions.showCursor);
            if (this.launchOptions.recordFailedRequests) {
                const additionalFailedStatus = this.launchOptions.additionalFailedStatus || [];
                yield action.recordFailedRequests(this.page, additionalFailedStatus, (request) => this.failedRequests.push(request));
            }
            if (this.launchOptions.recordPageErrors) {
                yield action.recordPageErrors(this.page, (err) => this.pageErrors.push(err));
            }
            if (this.launchOptions.emulateDevice) {
                yield action.emulateDevice(this.launchOptions.emulateDevice, this.page);
            }
        });
    }
    getInstances() {
        if (this.browser && this.page) {
            return [this.browser, this.page];
        }
        return [undefined, undefined];
    }
    initWith(options) {
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield this.launchAction(options); }));
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield this.startNewPage(); }));
        return this;
    }
    recordPageErrors() {
        if (this.page === undefined) {
            this.launchOptions.recordPageErrors = true;
            return this;
        }
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.recordPageErrors(this.page, (err) => this.pageErrors.push(err)); }));
        return this;
    }
    recordFailedRequests(...additionalFailedStatus) {
        if (this.page === undefined) {
            this.launchOptions.recordFailedRequests = true;
            this.launchOptions.additionalFailedStatus = additionalFailedStatus || [];
            return this;
        }
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
            return yield action.recordFailedRequests(this.page, additionalFailedStatus || [], (request) => this.failedRequests.push(request));
        }));
        return this;
    }
    recordRequestsTo(partialUrl) {
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
            return yield action.recordRequestsTo(partialUrl, this.page, (request) => this.sentRequests.push(request));
        }));
        return this;
    }
    navigateTo(url, options = actions_1.defaultNavigationOptions) {
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.navigateTo(url, options, this.page); }));
        return this;
    }
    wait(durationInMilliseconds) {
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.wait(durationInMilliseconds, this.page); }));
        return this;
    }
    click(selector, options = actions_1.defaultClickOptions) {
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.click(selector, options, this.page); }));
        return this;
    }
    clear(selector, options = actions_1.defaultClickOptions) {
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.clear(selector, options, this.page); }));
        return this;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    find(selector, waitOptions = actions_1.defaultWaitOptions) {
        return {
            // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
            withText: (text) => {
                return {
                    click: (clickOptions = actions_1.defaultClickOptions) => {
                        const mergedOptions = Object.assign(Object.assign(Object.assign(Object.assign({}, actions_1.defaultWaitOptions), actions_1.defaultClickOptions), waitOptions), clickOptions);
                        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.clickOnSelectorWithText(selector, text, mergedOptions, this.page); }));
                        return this;
                    },
                };
            },
            // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
            withExactText: (text) => {
                return {
                    click: (clickOptions = actions_1.defaultClickOptions) => {
                        const mergedOptions = Object.assign(Object.assign(Object.assign(Object.assign({}, actions_1.defaultWaitOptions), actions_1.defaultClickOptions), waitOptions), clickOptions);
                        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.clickOnSelectorWithExactText(selector, text, mergedOptions, this.page); }));
                        return this;
                    },
                };
            },
        };
    }
    hover(selector, options = actions_1.defaultHoverOptions) {
        const mergedOptions = Object.assign(Object.assign({}, actions_1.defaultHoverOptions), options);
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.hover(selector, mergedOptions, this.page); }));
        return this;
    }
    typeText(text, options = actions_1.defaultTypeTextOptions) {
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.typeText(text, options, this.page); }));
        return this;
    }
    /**
     * Paste text in the element that is currently active (that has the focus) on the page.
     * If the element does not handle the 'paste' event through an explicit event handler,
     * then you must set the option {handlePasteEvent: true}.
     * When you set this option to true, the controller will automatically attach a basic 'paste' event handler.
     * @param {string} text
     * @param {PasteOptions} [options=defaultPasteOptions]
     * @returns {PuppeteerController}
     * @memberof PuppeteerController
     */
    pasteText(text, options = actions_1.defaultPasteOptions) {
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.pasteTextInActiveElement(text, options, this.page); }));
        return this;
    }
    pressKey(key, options = actions_1.defaultKeyboardPressOptions) {
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.pressKey(key, options, this.page); }));
        return this;
    }
    select(...values) {
        return {
            in: (selector, options = actions_1.defaultSelectOptions) => {
                this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.select(selector, values, options, this.page); }));
                return this;
            },
        };
    }
    close() {
        if (this.isExecutingActions) {
            throw new Error('Error: there are some pending actions. You should call the close() method at a later time');
        }
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield action.closeBrowser(this.browser); }));
        return this;
    }
    runStory(story, param) {
        if (typeof story !== 'function') {
            throw new Error('Error: story should be a function');
        }
        if (param === undefined) {
            this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield story(this); }));
            return this;
        }
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () { return yield story(this, param); }));
        return this;
    }
    /**
     * Try to maximize the browser window size.
     * This method might not work on CI environment.
     * In this case setup the minViewPort parameter to ensure the page has a minimum width and height.
     * @param {MinViewPort} [minViewPort]
     * @returns {PuppeteerController}
     * @memberof PuppeteerController
     */
    withMaxSizeWindow(minViewPort) {
        this.launchOptions.browserWindowShouldBeMaximized = true;
        this.launchOptions.minViewPort = minViewPort;
        return this;
    }
    withCursor() {
        this.launchOptions.showCursor = true;
        return this;
    }
    emulateDevice(deviceName) {
        const device = actions_1.getDevice(deviceName);
        if (!device) {
            throw new Error(`device '${deviceName}' is unknown. It must be one of : [${actions_1.allKnownDevices
                .map((d) => d.name)
                .join(';')}] `);
        }
        this.launchOptions.emulateDevice = device;
        return this;
    }
    getCurrentUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield action.getCurrentUrl(this.page);
        });
    }
    getCurrentBrowserWindowState() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield action.getCurrentBrowserWindowState(this.page);
            return result;
        });
    }
    getValueOf(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield action.getValueOf(selector, this.page);
            return result;
        });
    }
    getClassListOf(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield action.getClassListOf(selector, this.page);
            return result;
        });
    }
    getInnerTextOf(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield action.getInnerTextOf(selector, this.page);
            return result;
        });
    }
    isChecked(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield action.isChecked(selector, this.page);
        });
    }
    hasFocus(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield action.hasFocus(selector, this.page);
        });
    }
    isDisabled(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield action.isDisabled(selector, this.page);
        });
    }
    isVisible(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield action.isVisible(selector, this.page);
        });
    }
    isNotVisible(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            return !(yield action.isVisible(selector, this.page));
        });
    }
    hasClass(selector, className) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield action.hasClass(selector, className, this.page);
        });
    }
    hasAttributeWithValue(selector, attributeName, attributeValue) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield action.hasAttributeWithValue(selector, attributeName, attributeValue, this.page);
        });
    }
    hasExactValue(selector, value) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield action.hasExactValue(selector, value, this.page);
        });
    }
    hasText(selector, text) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield action.hasText(selector, text, this.page);
        });
    }
    getSelectedOptionOf(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield action.getSelectedOptionOf(selector, this.page);
            return result;
        });
    }
    getAllOptionsOf(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield action.getAllOptionsOf(selector, this.page);
            return result;
        });
    }
    getComputedStyleOf(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield action.getComputedStyleOf(selector, this.page);
            return result;
        });
    }
    getClientRectangleOf(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield action.getClientRectangleOf(selector, this.page);
            return result;
        });
    }
    stringifyRequest(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield utils_1.stringifyRequest(request);
            return result;
        });
    }
    safeExecutePredicate(predicate) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield predicate();
                return result;
            }
            catch (error) {
                return false;
            }
        });
    }
    assertFor(predicate, errorMessage, options = exports.defaultAssertOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.page) {
                throw new Error('Error: expect statement only works when a page has been opened.');
            }
            const timeout = options.timeoutInMilliseconds;
            const interval = 100;
            const nbIntervals = timeout / interval;
            const stabilityCounterMaxValue = options.stabilityInMilliseconds / interval;
            let stabilityCounterCurrentValue = 0;
            for (let index = 0; index < nbIntervals; index++) {
                yield this.page.waitFor(interval);
                const result = yield this.safeExecutePredicate(predicate);
                if (result === true) {
                    stabilityCounterCurrentValue += 1;
                }
                if (result === false) {
                    stabilityCounterCurrentValue = 0;
                }
                if (stabilityCounterCurrentValue >= stabilityCounterMaxValue) {
                    return;
                }
            }
            if (typeof errorMessage === 'string') {
                throw new Error(errorMessage);
            }
            throw new Error(yield errorMessage());
        });
    }
    expectThat(selector) {
        return {
            hasAttribute: (attributeName) => {
                return {
                    withValue: (attributeValue, options = exports.defaultAssertOptions) => {
                        const assertOptions = Object.assign(Object.assign({}, exports.defaultAssertOptions), options);
                        this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
                            yield this.assertFor(() => __awaiter(this, void 0, void 0, function* () { return yield this.hasAttributeWithValue(selector, attributeName, attributeValue); }), () => __awaiter(this, void 0, void 0, function* () {
                                const hasAttributeWithValue = yield this.hasAttributeWithValue(selector, attributeName, attributeValue);
                                if (hasAttributeWithValue) {
                                    const unexpectedError = `Error: selector '${selector}' got the attribute '${attributeName}' with the value '${attributeValue}' after provided timeout. You should increase the timeout value`;
                                    return unexpectedError;
                                }
                                const errorMessage = `Error: Selector '${selector}' does not have the attribute '${attributeName}' with the value '${attributeValue}'.`;
                                return errorMessage;
                            }), assertOptions);
                        }));
                        return this;
                    },
                };
            },
            hasExactValue: (value, options = exports.defaultAssertOptions) => {
                const assertOptions = Object.assign(Object.assign({}, exports.defaultAssertOptions), options);
                this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.assertFor(() => __awaiter(this, void 0, void 0, function* () { return yield this.hasExactValue(selector, value); }), () => __awaiter(this, void 0, void 0, function* () {
                        const currentValue = yield this.getValueOf(selector);
                        let currentValueAsString = currentValue;
                        if (currentValue === undefined) {
                            currentValueAsString = 'undefined';
                        }
                        if (currentValue === null) {
                            currentValueAsString = 'null';
                        }
                        const errorMessage = `Error: Selector '${selector}' current value is: '${currentValueAsString}', but this does not match the expected value: '${value}'`;
                        return errorMessage;
                    }), assertOptions);
                }));
                return this;
            },
            hasClass: (className, options = exports.defaultAssertOptions) => {
                const assertOptions = Object.assign(Object.assign({}, exports.defaultAssertOptions), options);
                this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.assertFor(() => __awaiter(this, void 0, void 0, function* () { return yield this.hasClass(selector, className); }), () => __awaiter(this, void 0, void 0, function* () {
                        const currentClassList = yield this.getClassListOf(selector);
                        const currentClassListAsString = `[${currentClassList.join(', ')}]`;
                        const errorMessage = `Error: Selector '${selector}' current class list is: ${currentClassListAsString}, but this list does not contain the expected value: '${className}'.`;
                        return errorMessage;
                    }), assertOptions);
                }));
                return this;
            },
            hasFocus: (options = exports.defaultAssertOptions) => {
                const assertOptions = Object.assign(Object.assign({}, exports.defaultAssertOptions), options);
                this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.assertFor(() => __awaiter(this, void 0, void 0, function* () { return yield this.hasFocus(selector); }), () => __awaiter(this, void 0, void 0, function* () {
                        const hasCurrentFocus = yield this.hasFocus(selector);
                        if (hasCurrentFocus) {
                            const unexpectedError = `Error: selector '${selector}' got focus after provided timeout. You should increase the timeout value`;
                            return unexpectedError;
                        }
                        const errorMessage = `Error: selector '${selector}' does not have the focus.`;
                        return errorMessage;
                    }), assertOptions);
                }));
                return this;
            },
            hasText: (text, options = exports.defaultAssertOptions) => {
                const assertOptions = Object.assign(Object.assign({}, exports.defaultAssertOptions), options);
                this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.assertFor(() => __awaiter(this, void 0, void 0, function* () { return yield this.hasText(selector, text); }), () => __awaiter(this, void 0, void 0, function* () {
                        const currentInnerText = yield this.getInnerTextOf(selector);
                        let currentValueAsString = currentInnerText;
                        if (currentInnerText === undefined) {
                            currentValueAsString = 'undefined';
                        }
                        if (currentInnerText === null) {
                            currentValueAsString = 'null';
                        }
                        const errorMessage = `Error: Selector '${selector}' current innerText is: '${currentValueAsString}', but this does not match the expected text: '${text}'`;
                        return errorMessage;
                    }), assertOptions);
                }));
                return this;
            },
            isDisabled: (options = exports.defaultAssertOptions) => {
                const assertOptions = Object.assign(Object.assign({}, exports.defaultAssertOptions), options);
                this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.assertFor(() => __awaiter(this, void 0, void 0, function* () { return yield this.isDisabled(selector); }), () => __awaiter(this, void 0, void 0, function* () {
                        const isDisabled = yield this.isDisabled(selector);
                        if (isDisabled) {
                            const unexpectedError = `Error: selector '${selector}' was disabled after provided timeout. You should increase the timeout value`;
                            return unexpectedError;
                        }
                        const errorMessage = `Error: Selector '${selector}' is not disabled.`;
                        return errorMessage;
                    }), assertOptions);
                }));
                return this;
            },
            isEnabled: (options = exports.defaultAssertOptions) => {
                const assertOptions = Object.assign(Object.assign({}, exports.defaultAssertOptions), options);
                this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.assertFor(() => __awaiter(this, void 0, void 0, function* () { return !(yield this.isDisabled(selector)); }), () => __awaiter(this, void 0, void 0, function* () {
                        const isEnabled = !(yield this.isDisabled(selector));
                        if (isEnabled) {
                            const unexpectedError = `Error: selector '${selector}' was enabled after provided timeout. You should increase the timeout value`;
                            return unexpectedError;
                        }
                        const errorMessage = `Error: Selector '${selector}' is disabled.`;
                        return errorMessage;
                    }), assertOptions);
                }));
                return this;
            },
            isVisible: (options = exports.defaultAssertOptions) => {
                const assertOptions = Object.assign(Object.assign({}, exports.defaultAssertOptions), options);
                this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
                    const errorMessage = `Error: selector '${selector}' is not visible.`;
                    yield this.assertFor(() => __awaiter(this, void 0, void 0, function* () { return yield this.isVisible(selector); }), errorMessage, assertOptions);
                }));
                return this;
            },
            isNotVisible: (options = exports.defaultAssertOptions) => {
                const assertOptions = Object.assign(Object.assign({}, exports.defaultAssertOptions), options);
                this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
                    const errorMessage = `Error: selector '${selector}' is visible.`;
                    yield this.assertFor(() => __awaiter(this, void 0, void 0, function* () { return yield this.isNotVisible(selector); }), errorMessage, assertOptions);
                }));
                return this;
            },
        };
    }
    waitFor(predicate, errorMessage, options = exports.defaultWaitUntilOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.page) {
                throw new Error('Error: expect statement only works when a page has been opened.');
            }
            utils_1.report('wait options:', options.verbose);
            utils_1.report(JSON.stringify(options, null, 2), options.verbose);
            const timeout = options.timeoutInMilliseconds;
            const interval = 100;
            const nbIntervals = timeout / interval;
            const stabilityCounterMaxValue = options.stabilityInMilliseconds / interval;
            let stabilityCounterCurrentValue = 0;
            for (let index = 0; index < nbIntervals; index++) {
                yield this.page.waitFor(interval);
                const result = yield predicate();
                utils_1.report(`predicate returned ${result} after waiting ${(index + 1) * interval} ms`, options.verbose);
                if (result === true) {
                    stabilityCounterCurrentValue += 1;
                }
                if (result === false) {
                    stabilityCounterCurrentValue = 0;
                }
                if (stabilityCounterCurrentValue >= stabilityCounterMaxValue) {
                    utils_1.report(`predicate always returned ${result} during ${options.stabilityInMilliseconds} ms: no need to wait anymore.`, options.verbose);
                    return;
                }
            }
            utils_1.report(`predicate could not converge to a stable result or always returned false during ${options.timeoutInMilliseconds} ms.`, options.verbose);
            if (!options.throwOnTimeout) {
                utils_1.report('exiting waiting mechanism without throwing', options.verbose);
                return;
            }
            if (typeof errorMessage === 'string') {
                throw new Error(errorMessage);
            }
            throw new Error(yield errorMessage());
        });
    }
    waitUntil(predicate, options = exports.defaultWaitUntilOptions, errorMessage) {
        const waitUntilOptions = Object.assign(Object.assign({}, exports.defaultWaitUntilOptions), options);
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
            const defaultErrorMessage = `predicate still resolved to false after ${waitUntilOptions.timeoutInMilliseconds} ms.`;
            yield this.waitFor(predicate, errorMessage || defaultErrorMessage, waitUntilOptions);
        }));
        return this;
    }
    waitForStability(getValue, errorMessage, options = exports.defaultWaitUntilOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.page) {
                throw new Error('Error: expect statement only works when a page has been opened.');
            }
            const timeout = options.timeoutInMilliseconds;
            const interval = 100;
            const nbIntervals = timeout / interval;
            const stabilityCounterMaxValue = options.stabilityInMilliseconds / interval;
            let stabilityCounterCurrentValue = 0;
            let previousValue = yield getValue();
            for (let index = 0; index < nbIntervals; index++) {
                yield this.page.waitFor(interval);
                const currentValue = yield getValue();
                if (currentValue !== previousValue) {
                    previousValue = currentValue;
                    stabilityCounterCurrentValue = 0;
                    continue;
                }
                if (currentValue === previousValue) {
                    stabilityCounterCurrentValue += 1;
                }
                if (stabilityCounterCurrentValue >= stabilityCounterMaxValue) {
                    return;
                }
            }
            if (!options.throwOnTimeout) {
                return;
            }
            if (typeof errorMessage === 'string') {
                throw new Error(errorMessage);
            }
            throw new Error(yield errorMessage());
        });
    }
    /**
     * Waits until the function getValue() returns the same result during a specified period of time.
     *
     * @param {(() => Promise<string | number>)} getValue async function that should return either a string or or number.
     * @param {Partial<WaitUntilOptions>} [options=defaultWaitUntilOptions]
     * @param {(string | (() => Promise<string>))} [errorMessage] error message to be returned when the throw option is activated
     * @returns {PuppeteerController}
     * @memberof PuppeteerController
     */
    waitForStabilityOf(getValue, options = exports.defaultWaitUntilOptions, errorMessage) {
        const waitUntilOptions = Object.assign(Object.assign({}, exports.defaultWaitUntilOptions), options);
        this.actions.push(() => __awaiter(this, void 0, void 0, function* () {
            const defaultErrorMessage = `The value function cannot converge to a stable result after ${waitUntilOptions.timeoutInMilliseconds} ms.`;
            yield this.waitForStability(getValue, errorMessage || defaultErrorMessage, waitUntilOptions);
        }));
        return this;
    }
    takeFullPageScreenshotAsBase64(options = actions_1.mandatoryFullPageScreenshotOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield action.takeFullPageScreenshotAsBase64(options, this.page);
            return result;
        });
    }
    /**
     * Create a Selector object to be able to target a DOM element
     * that is embedded in a complex dom hierarchy or dom array
     *
     * @param {string} selector
     * @returns {SelectorController}
     * @memberof PuppeteerController
     */
    selector(selector) {
        return new selector_1.SelectorController(selector, this);
    }
}
exports.PuppeteerController = PuppeteerController;
/**
 * cast input object as a PuppeteerController instance
 * usefull when such instance is store in an untyped context
 * @param pptcInstance : an untyped PuppeteerController instance
 */
exports.cast = (pptcInstance) => {
    return pptcInstance;
};
